/* file: algorithm_base.h */
/*******************************************************************************
* Copyright 2014-2019 Intel Corporation.
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
//++
//  Implementation of base classes defining algorithm interface.
//--
*/

#ifndef __ALGORITHM_BASE_H__
#define __ALGORITHM_BASE_H__

#include "services/env_detect.h"
#include "algorithms/algorithm_container_base_common.h"
#include "algorithms/algorithm_container_base_batch.h"
#include "algorithms/algorithm_base_common.h"
#include "algorithms/algorithm_base_mode.h"
#include "algorithms/algorithm_base_mode_impl.h"
#include "algorithms/algorithm_base_mode_batch.h"
#include "algorithms/algorithm_base_mode_online.h"
#include "algorithms/algorithm_base_mode_distributed.h"

#endif
